#!/usr/bin/python
# vim:fileencoding=utf-8
#
# Walk a domain that's using NSEC and print in zonefile format.

import sys
import ldnsx

def walk(domain):
        res = ldnsx.resolver("193.110.157.136", dnssec=True)
        pkt = res.query(domain, 666)
        try:
                nsec_rr = pkt.authority(rr_type="NSEC")[0]
        except:
                print "no NSEC found, domain is not signed or using NSEC3"
                sys.exit()
        for rr_type in nsec_rr[5].split(' ')[:-1]:
                for rr in ldnsx.get_rrs(domain, rr_type):
                        print str(rr)[:-1]
        next_rec = nsec_rr[4]
        if (next_rec != domain) and (next_rec[-len(domain):] == domain):
                walk(next_rec)

walk("xelerance.com")




#!/usr/bin/python
# vim:fileencoding=utf-8
#
# AXFR client with IDN (Internationalized Domain Names) support
#

import ldns
import encodings.idna

def utf2name(name):
    return '.'.join([encodings.idna.ToASCII(a) for a in name.split('.')])
def name2utf(name):
    return '.'.join([encodings.idna.ToUnicode(a) for a in name.split('.')])

resolver = ldnsx.resolver("zone.nic.cz")

#Print results
for rr in resolver.AXFR(utf2name(u"háčkyčárky.cz")):
   # rdf = rr.owner()
   # if (rdf.get_type() == ldns.LDNS_RDF_TYPE_DNAME):
   #     print "RDF owner: type=",rr.type(),"data=",name2utf(rr.owner())
   # else:
   #     print "RDF owner: type=",rdf.get_type_str(),"data=",str(rdf)
   # print "   RR type=", rr.get_type_str()," ttl=",rr.ttl()
   # for rdf in rr.rdfs():
   #     if (rdf.get_type() == ldns.LDNS_RDF_TYPE_DNAME):
   #         print "   RDF: type=",rdf.get_type_str(),"data=",name2utf(str(rdf))
   #     else:
   #         print "   RDF: type=",rdf.get_type_str(),"data=",str(rdf)




#!/usr/bin/python
# -*- coding: utf-8 -*-
import ldns
import sys

debug = True

# Check args
argc = len(sys.argv)
name = "www.nic.cz"
if argc < 2:
   print "Usage:", sys.argv[0], "domain [resolver_addr]"
   sys.exit(1)
else:
   name = sys.argv[1]

# Create resolver
resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
resolver.set_dnssec(True)

# Custom resolver
if argc > 2:
   # Clear previous nameservers
   ns = resolver.pop_nameserver()
   while ns != None:
      ns = resolver.pop_nameserver()
   ip = ldns.ldns_rdf.new_frm_str(sys.argv[2], ldns.LDNS_RDF_TYPE_A)
   resolver.push_nameserver(ip)

# Resolve DNS name
pkt = resolver.query(name, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
if pkt and pkt.answer():

   # Debug
   if debug:
      print "NS returned:", pkt.get_rcode(), "(AA: %d AD: %d)" % ( pkt.ad(), pkt.ad() )

   # SERVFAIL indicated bogus name
   if pkt.get_rcode() is ldns.LDNS_RCODE_SERVFAIL:
      print name, "is bogus"

   # Check AD (Authenticated) bit
   if pkt.get_rcode() is ldns.LDNS_RCODE_NOERROR:
      if pkt.ad(): print name, "is secure"
      else:        print name, "is insecure"



#!/usr/bin/python
import ldns

#Read zone from file
zone = ldns.ldns_zone.new_frm_fp(open("zone.txt","r"), None, 0, ldns.LDNS_RR_CLASS_IN)
print zone

print "SOA:", zone.soa()
for r in zone.rrs().rrs():
   print "RR:", r


zone = ldns.ldns_zone()
#print zone




#!/usr/bin/env python

#
# ldns_pkt testing script.
#
# Do not use constructs that differ between Python 2 and 3.
# Use write on stdout or stderr.
#


import ldns
import sys
import os
import inspect


class_name = "ldns_pkt"
method_name = None
error_detected = False
temp_fname = "tmp_pkt.txt"


def set_error():
    """
        Writes an error message and sets error flag.
    """
    global class_name
    global method_name
    global error_detected
    error_detected = True
    sys.stderr.write("(line %d): malfunctioning method %s.\n" % \
       (inspect.currentframe().f_back.f_lineno, method_name))


#if not error_detected:
if True:
    method_name = class_name + ".aa()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AA)
    try:
        ret = pkt.aa()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR)
    try:
        ret = pkt.aa()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".ad()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.ad()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR)
    try:
        ret = pkt.ad()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".additional()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.additional()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
        if ret.rr_count() != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".algorithm2str()"
    try:
        ret = ldns.ldns_pkt.algorithm2str(ldns.LDNS_DSA)
        if not isinstance(ret, str):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".all()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.all()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
        if ret.rr_count() != 1:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".all_noquestion()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.all_noquestion()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
        if ret.rr_count() != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".ancount()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.ancount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".answer()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.answer()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
        if ret.rr_count() != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".answerfrom()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.answerfrom()
        if ret != None:
            set_error()
    except:
        set_error()
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    pkt = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
    try:
        ret = pkt.answerfrom()
        if not isinstance(ret, ldns.ldns_rdf):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".arcount()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.arcount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".authority()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.authority()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
        if ret.rr_count() != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".cd()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_CD)
    try:
        ret = pkt.cd()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR)
    try:
        ret = pkt.cd()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".cert_algorithm2str()"
    try:
        ret = ldns.ldns_pkt.cert_algorithm2str(ldns.LDNS_CERT_PGP)
        if not isinstance(ret, str):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".clone()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.clone()
        if not isinstance(ret, ldns.ldns_pkt):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".ends()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.edns()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_dnssec(True)
    pkt = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
    try:
        ret = pkt.edns()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".ends_data()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.edns_data()
        if ret != None:
            set_error()
    except:
        set_error()
    #resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    #resolver.set_dnssec(True)
    #pkt = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
    #try:
    #    ret = pkt.edns_data()
    #    print ret
    #    if not isinstance(ret, ldns.ldns_rdf):
    #        set_error()
    #    if ret != True:
    #        set_error()
    #except:
    #    set_error()


#if not error_detected:
if True:
    method_name = class_name + ".edns_do()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.edns_do()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".edns_extended_rcode()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.edns_extended_rcode()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()
    #resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    #resolver.set_dnssec(True)
    #pkt = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
    #try:
    #    ret = pkt.edns_extended_rcode()
    #    if (not isinstance(ret, int)) and (not isinstance(ret, long)):
    #        set_error()
    #    if ret != 0:
    #        set_error()
    #except:
    #    set_error()


#if not error_detected:
if True:
    method_name = class_name + ".edns_udp_size()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.edns_udp_size()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_dnssec(True)
    pkt = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
    try:
        ret = pkt.edns_udp_size()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret == 0: # Don't know the actual size, but must be greater than 0.
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".edns_version()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.edns_version()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()
    #resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    #resolver.set_dnssec(True)
    #pkt = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
    #try:
    #    ret = pkt.edns_version()
    #    if (not isinstance(ret, int)) and (not isinstance(ret, long)):
    #        set_error()
    #    if ret != 0:
    #        set_error()
    #except:
    #    set_error()


#if not error_detected:
if True:
    method_name = class_name + ".edns_z()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.edns_z()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".empty()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.empty()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".get_opcode()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.get_opcode()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != ldns.LDNS_PACKET_QUERY:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".get_rcode()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.get_rcode()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != ldns.LDNS_RCODE_NOERROR:
            set_error()
    except:
        set_error()
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_dnssec(True)
    pkt = resolver.query("nonexistent_domain.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN)
    try:
        ret = pkt.get_rcode()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != ldns.LDNS_RCODE_NXDOMAIN:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".get_section_clone()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.get_section_clone(ldns.LDNS_SECTION_ANY)
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        ret = pkt.get_section_clone(ldns.LDNS_SECTION_ANSWER)
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        ret = pkt.get_section_clone("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".id()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.id()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".new()"
    try:
        pkt = ldns.ldns_pkt.new()
        if not isinstance(pkt, ldns.ldns_pkt):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".new_query()"
    dname = ldns.ldns_dname("test.nic.cz.")
    try:
        pkt = ldns.ldns_pkt.new_query(dname, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
        if not isinstance(pkt, ldns.ldns_pkt):
            set_error()
    except:
        set_error()
    rdf = ldns.ldns_rdf_new_frm_str(ldns.LDNS_RDF_TYPE_DNAME, "test.nic.cz.")
    try:
        pkt = ldns.ldns_pkt.new_query(rdf, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
        if not isinstance(pkt, ldns.ldns_pkt):
            set_error()
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query("bad argument", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query(dname, "bad argument", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query(dname, ldns.LDNS_RR_TYPE_A, "bad argument", ldns.LDNS_QR | ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query(dname, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".new_query_frm_str()"
    try:
        pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz", ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AA)
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query_frm_str(pkt, ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AA)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz", "bad argument", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AA)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz", ldns.LDNS_RR_TYPE_ANY, "bad argument", ldns.LDNS_QR | ldns.LDNS_AA)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz", ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".nscount()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.nscount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".opcode2str()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        ret = pkt.opcode2str()
        if not isinstance(ret, str):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".print_to_file()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    f = open(temp_fname, "w")
    try:
        pkt.print_to_file(f)
    except:
        set_error()
    f.close()
    f = open(temp_fname, "r")
    if len(f.readlines()) != 14:
        set_error()
    f.close()
    os.remove(temp_fname)


#if not error_detected:
if True:
    method_name = class_name + ".push_rr()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    try:
        ret = pkt.push_rr(ldns.LDNS_SECTION_ANSWER, rr)
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.push_rr("bad argument", rr)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.push_rr(ldns.LDNS_SECTION_ANSWER, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".push_rr_list()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    rrl.push_rr(rr)
    rrl.push_rr(rr)
    try:
        ret = pkt.push_rr_list(ldns.LDNS_SECTION_ANSWER, rrl)
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.push_rr_list("bad argument", rrl)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.push_rr_list(ldns.LDNS_SECTION_ANSWER, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".qdcount()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.qdcount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 1:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".qr()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AA)
    try:
        ret = pkt.qr()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_AA)
    try:
        ret = pkt.qr()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".querytime()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.querytime()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".question()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AD)
    try:
        ret = pkt.question()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
        if ret.rr_count() != 1:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".ra()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RA)
    try:
        ret = pkt.ra()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR)
    try:
        ret = pkt.ra()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".rcode2str()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RA)
    try:
        ret = pkt.rcode2str()
        if not isinstance(ret, str):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".rd()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
    try:
        ret = pkt.rd()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR)
    try:
        ret = pkt.rd()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".reply_type()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
    try:
        ret = pkt.reply_type()
        if ret != ldns.LDNS_PACKET_ANSWER:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".rr()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    pkt.push_rr(ldns.LDNS_SECTION_ANSWER, rr)
    try:
        ret = pkt.rr(ldns.LDNS_SECTION_ANSWER, rr)
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.rr(ldns.LDNS_SECTION_QUESTION, rr)
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.rr("bad argument", rr)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.rr(ldns.LDNS_SECTION_QUESTION, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".rr_list_by_name()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    pkt.push_rr(ldns.LDNS_SECTION_ANSWER, rr)
    rdf = ldns.ldns_rdf_new_frm_str(ldns.LDNS_RDF_TYPE_DNAME, "test1")
    try:
        ret = pkt.rr_list_by_name(rdf, ldns.LDNS_SECTION_ANSWER)
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_name(rdf, ldns.LDNS_SECTION_QUESTION)
        if ret != None:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_name("bad argument", ldns.LDNS_SECTION_ANSWER)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_name(rdf, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".rr_list_by_name_and_type()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz.", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    pkt.push_rr(ldns.LDNS_SECTION_ANSWER, rr)
    rdf = ldns.ldns_rdf_new_frm_str(ldns.LDNS_RDF_TYPE_DNAME, "test1")
    try:
        ret = pkt.rr_list_by_name_and_type(rdf, ldns.LDNS_RR_TYPE_A, ldns.LDNS_SECTION_ANSWER)
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_name_and_type(rdf, ldns.LDNS_RR_TYPE_AAAA, ldns.LDNS_SECTION_ANSWER)
        if ret != None:
            set_error()
    except:
        set_error()
    #try:
    #    ret = pkt.rr_list_by_name_and_type("bad argument", ldns.LDNS_RR_TYPE_A, ldns.LDNS_SECTION_ANSWER)
    #    set_error()
    #except TypeError as e:
    #    pass
    #except:
    #    set_error()
    try:
        ret = pkt.rr_list_by_name_and_type(rdf, "bad argument", ldns.LDNS_SECTION_ANSWER)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_name_and_type(rdf, ldns.LDNS_RR_TYPE_A, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".rr_list_by_type()"
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz.", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_RD)
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    pkt.push_rr(ldns.LDNS_SECTION_ANSWER, rr)
    try:
        ret = pkt.rr_list_by_type(ldns.LDNS_RR_TYPE_A, ldns.LDNS_SECTION_ANSWER)
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_type(ldns.LDNS_RR_TYPE_AAAA, ldns.LDNS_SECTION_ANSWER)
        if ret != None:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_type("bad argument", ldns.LDNS_SECTION_ANSWER)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.rr_list_by_type(ldns.LDNS_RR_TYPE_A, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".safe_push_rr()"
    pkt = ldns.ldns_pkt.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    try:
        ret = pkt.safe_push_rr(ldns.LDNS_SECTION_ANSWER, rr)
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.safe_push_rr(ldns.LDNS_SECTION_ANSWER, rr)
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.safe_push_rr("bad argument", rr)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.safe_push_rr(ldns.LDNS_SECTION_ANSWER, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".safe_push_rr_list()"
    pkt = ldns.ldns_pkt.new()
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    rrl.push_rr(rr)
    try:
        ret = pkt.safe_push_rr_list(ldns.LDNS_SECTION_ANSWER, rrl)
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.safe_push_rr_list(ldns.LDNS_SECTION_ANSWER, rrl)
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.safe_push_rr_list("bad argument", rrl)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = pkt.safe_push_rr_list(ldns.LDNS_SECTION_ANSWER, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_aa()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_aa(True)
        if pkt.aa() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_aa(False)
        if pkt.aa() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_ad()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_ad(True)
        if pkt.ad() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_ad(False)
        if pkt.ad() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_additional()"
    pkt = ldns.ldns_pkt.new()
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    rrl.push_rr(rr)
    try:
        pkt.set_additional(rrl)
        if not isinstance(pkt.additional() , ldns.ldns_rr_list):
            set_error()
        if pkt.additional() != rrl:
            set_error()
    except:
        set_error()
    try:
        pkt.set_additional("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_ancount()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_ancount(1)
        ret = pkt.ancount()
        if ret != 1:
            set_error()
    except:
        set_error()
    try:
        pkt.set_ancount("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_answer()"
    pkt = ldns.ldns_pkt.new()
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    rrl.push_rr(rr)
    try:
        pkt.set_answer(rrl)
        if not isinstance(pkt.additional() , ldns.ldns_rr_list):
            set_error()
        if pkt.answer() != rrl:
            set_error()
    except:
        set_error()
    try:
        pkt.set_answer("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_answerfrom()"
    pkt = ldns.ldns_pkt.new()
    rdf = ldns.ldns_rdf_new_frm_str(ldns.LDNS_RDF_TYPE_A, "127.0.0.1")
    try:
        pkt.set_answerfrom(rdf)
        ret = pkt.answerfrom()
        if ret != rdf:
            set_error()
    except:
        set_error()
    try:
        pkt.set_answerfrom("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_arcount()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_arcount(1)
        ret = pkt.arcount()
        if ret != 1:
            set_error()
    except:
        set_error()
    try:
        pkt.set_arcount("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_authority()"
    pkt = ldns.ldns_pkt.new()
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    rrl.push_rr(rr)
    try:
        pkt.set_authority(rrl)
        if not isinstance(pkt.additional() , ldns.ldns_rr_list):
            set_error()
        if pkt.authority() != rrl:
            set_error()
    except:
        set_error()
    try:
        pkt.set_authority("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_cd()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_cd(True)
        if pkt.cd() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_cd(False)
        if pkt.cd() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_edns_data()"
    pkt = ldns.ldns_pkt.new()
    rdf = ldns.ldns_rdf_new_frm_str(ldns.LDNS_RDF_TYPE_A, "127.0.0.1")
    try:
        pkt.set_edns_data(rdf)
        ret = pkt.edns_data()
        if ret != rdf:
            set_error()
    except:
        set_error()
    try:
        pkt.set_edns_data("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_edns_do()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_edns_do(True)
        if pkt.edns_do() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_edns_do(False)
        if pkt.edns_do() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_edns_extended_rcode()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_edns_extended_rcode(8)
        ret = pkt.edns_extended_rcode()
        if ret != 8:
            set_error()
    except:
        set_error()
    try:
        pkt.set_edns_extended_rcode("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_edns_udp_size()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_edns_udp_size(4096)
        ret = pkt.edns_udp_size()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.set_edns_udp_size("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_edns_version()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_edns_version(8)
        ret = pkt.edns_version()
        if ret != 8:
            set_error()
    except:
        set_error()
    try:
        pkt.set_edns_version("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_edns_z()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_edns_z(4096)
        ret = pkt.edns_z()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.set_edns_z("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_flags()"
    pkt = ldns.ldns_pkt.new()
    try:
        ret = pkt.set_flags(ldns.LDNS_AA | ldns.LDNS_AD)
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_flags("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_id()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_id(4096)
        ret = pkt.id()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.set_id("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_nscount()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_nscount(1)
        ret = pkt.nscount()
        if ret != 1:
            set_error()
    except:
        set_error()
    try:
        pkt.set_nscount("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_opcode()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_opcode(ldns.LDNS_PACKET_QUERY)
        ret = pkt.get_opcode()
        if ret != ldns.LDNS_PACKET_QUERY:
            set_error()
    except:
        set_error()
    try:
        pkt.set_opcode("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_qdcount()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_qdcount(10)
        ret = pkt.qdcount()
        if ret != 10:
            set_error()
    except:
        set_error()
    try:
        pkt.set_qdcount("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_qr()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_qr(True)
        if pkt.qr() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_qr(False)
        if pkt.qr() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_querytime()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_querytime(65536)
        ret = pkt.querytime()
        if ret != 65536:
            set_error()
    except:
        set_error()
    try:
        pkt.set_querytime("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_question()"
    pkt = ldns.ldns_pkt.new()
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    rrl.push_rr(rr)
    try:
        pkt.set_question(rrl)
        if not isinstance(pkt.additional() , ldns.ldns_rr_list):
            set_error()
        if pkt.question() != rrl:
            set_error()
    except:
        set_error()
    try:
        pkt.set_question("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_ra()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_ra(True)
        if pkt.ra() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_ra(False)
        if pkt.ra() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_random_id()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_random_id()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_rcode()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_rcode(127)
        ret = pkt.get_rcode()
        if ret != 127:
            set_error()
    except:
        set_error()
    try:
        pkt.set_rcode("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_rd()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_rd(True)
        if pkt.rd() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_rd(False)
        if pkt.rd() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_section_count()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_section_count(ldns.LDNS_PACKET_QUESTION, 4096)
        ret = pkt.qdcount()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.set_section_count("bad argument", 4096)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        pkt.set_section_count(ldns.LDNS_PACKET_QUESTION, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_size()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_size(512)
        ret = pkt.size()
        if ret != 512:
            set_error()
    except:
        set_error()
    try:
        pkt.set_size("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_tc()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.set_tc(True)
        if pkt.tc() != True:
            set_error()
    except:
        set_error()
    try:
        pkt.set_tc(False)
        if pkt.tc() != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_timestamp()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".set_tsig()"
    pkt = ldns.ldns_pkt.new()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    try:
        pkt.set_tsig(rr)
        ret = pkt.tsig()
        if ret != rr:
            set_error()
    except:
        set_error()
    try:
        pkt.set_tsig("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".size()"
    pkt = ldns.ldns_pkt.new()
    pkt.set_size(512)
    try:
        ret = pkt.size()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 512:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".tc()"
    pkt = ldns.ldns_pkt.new()
    pkt.set_tc(True)
    try:
        ret = pkt.tc()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()
    pkt.set_tc(False)
    try:
        ret = pkt.tc()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".timestamp()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".tsig()"
    pkt = ldns.ldns_pkt.new()
    try:
        ret = pkt.tsig()
        if ret != None:
            set_error()
    except:
        set_error()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    pkt.set_tsig(rr)
    try:
        ret = pkt.tsig()
        if not isinstance(ret, ldns.ldns_rr):
            set_error()
        if ret != rr:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_pkt_tsig_add()"
    pkt = ldns.ldns_pkt.new()
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = pkt.update_pkt_tsig_add(resolver)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    try:
        ret = pkt.update_pkt_tsig_add("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_prcount()"
    pkt = ldns.ldns_pkt.new()
    try:
        ret = pkt.update_prcount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()
    pkt.update_set_prcount(127)
    try:
        ret = pkt.update_prcount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 127:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_set_adcount()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.update_set_adcount(4096)
        ret = pkt.update_ad()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.update_set_adcount("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_set_prcount()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.update_set_prcount(4096)
        ret = pkt.update_prcount()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.update_set_prcount("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_set_upcount()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.update_set_upcount(4096)
        ret = pkt.update_upcount()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.update_set_upcount("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_set_zo()"
    pkt = ldns.ldns_pkt.new()
    try:
        pkt.update_set_zo(4096)
        ret = pkt.update_zocount()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        pkt.update_set_zo("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_upcount()"
    pkt = ldns.ldns_pkt.new()
    try:
        ret = pkt.update_upcount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()
    pkt.update_set_upcount(127)
    try:
        ret = pkt.update_upcount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 127:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".update_zocount()"
    pkt = ldns.ldns_pkt.new()
    try:
        ret = pkt.update_zocount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()
    pkt.update_set_zo(127)
    try:
        ret = pkt.update_zocount()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 127:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".write_to_buffer()"
    pkt = pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_QR | ldns.LDNS_AA)
    buf = buf = ldns.ldns_buffer(4096)
    try:
        ret = pkt.write_to_buffer(buf)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()


if not error_detected:
    sys.stdout.write("%s: passed.\n" % (os.path.basename(__file__)))
else:
    sys.stdout.write("%s: errors detected.\n" % (os.path.basename(__file__)))
    sys.exit(1)



# Copyright (C) Xelerance Corp. <http://www.xelerance.com/>.
# Author: Christopher Olah <colah@xelerance.com>
# License: BSD

""" Easy DNS (including DNSSEC) via ldns.

ldns is a great library. It is a powerfull tool for
working with DNS. python-ldns it is a straight up clone of the C
interface, howver that is not a very good interface for python. Its
documentation is incomplete and some functions don't work as
described. And some objects don't have a full python API.

ldnsx aims to fix this. It wraps around the ldns python bindings,
working around its limitations and providing a well-documented, more
pythonistic interface.

**WARNING:** 

**API subject to change.** No backwards compatibility guarantee. Write software using this version at your own risk!

Examples
--------

Query the default resolver for google.com's A records. Print the response
packet.

>>> import ldnsx
>>> resolver = ldnsx.resolver()
>>> print resolver.query("google.com","A")


Print the root NS records from f.root-servers.net; if we get a
response, else an error message.

>>> import ldnsx
>>> pkt = ldnsx.resolver("f.root-servers.net").query(".", "NS")
>>> if pkt:
>>>    for rr in pkt.answer():
>>>       print rr
>>> else:
>>>    print "response not received" 

"""

import time, sys, calendar, warnings, socket
try:
	import ldns
except ImportError:
	print >> sys.stderr, "ldnsx requires the ldns-python sub-package from http://www.nlnetlabs.nl/projects/ldns/"
	print >> sys.stderr, "Fedora/CentOS: yum install ldns-python"
	print >> sys.stderr, "Debian/Ubuntu: apt-get install python-ldns"
	print >> sys.stderr, "openSUSE: zypper in python-ldns"
	sys.exit(1)

__version__ = "0.1"

def isValidIP(ipaddr):
	try:
		v4 = socket.inet_pton(socket.AF_INET,ipaddr)
		return 4
	except:
		try:
			v6 = socket.inet_pton(socket.AF_INET6,ipaddr)
			return 6
		except:
			return 0

def query(name, rr_type, rr_class="IN", flags=["RD"], tries = 3, res=None):
	"""Convenience function. Creates a resolver and then queries it. Refer to resolver.query() 
	   * name -- domain to query for 
	   * rr_type -- rr_type to query for
	   * flags -- flags for query (list of strings)
	   * tries -- number of times to retry the query on failure
	   * res -- configurations for the resolver as a dict -- see resolver()
	   """
	if isinstance(res, list) or isinstance(res, tuple):
		res = resolver(*res)
	elif isinstance(res, dict):
		res = resolver(**res)
	else:
		res = resolver(res)
	return res.query(name, rr_type, rr_class, flags, tries)

def get_rrs(name, rr_type, rr_class="IN", tries = 3, strict = False, res=None, **kwds):
	"""Convenience function. Gets RRs for name of type rr_type trying tries times. 
	   If strict, it raises and exception on failure, otherwise it returns []. 
	   * name -- domain to query for 
	   * rr_type -- rr_type to query for
	   * flags -- flags for query (list of strings)
	   * tries -- number of times to retry the query on failure
	   * strict -- if the query fails, do we return [] or raise an exception?
	   * res -- configurations for the resolver as a dict -- see resolver()
	   * kwds -- query filters, refer to packet.answer()
       """
	if isinstance(res, list) or isinstance(res, tuple):
		res = resolver(*res)
	elif isinstance(res, dict):
		res = resolver(**res)
	else:
		res = resolver(res)
	if "|" in rr_type:
		pkt = res.query(name, "ANY", rr_class=rr_class, tries=tries)
	else:
		pkt = res.query(name, rr_type, rr_class=rr_class, tries=tries)
	if pkt:
		if rr_type in ["", "ANY", "*"]:
			return pkt.answer( **kwds)
		else:
			return pkt.answer(rr_type=rr_type, **kwds)
	else:
		if strict:
			raise Exception("LDNS couldn't complete query")
		else:
			return []

def secure_query(name, rr_type, rr_class="IN", flags=["RD"], tries = 1, flex=False, res=None):
	"""Convenience function. Creates a resolver and then does a DNSSEC query. Refer to resolver.query() 
	   * name -- domain to query for 
	   * rr_type -- rr_type to query for
	   * flags -- flags for query (list of strings)
	   * tries -- number of times to retry the query on failure
	   * flex -- if we can't verify data, exception or warning?
	   * res -- configurations for the resolver as a dict -- see resolver()"""
	if isinstance(res, list) or isinstance(res, tuple):
		res = resolver(*res)
	elif isinstance(res, dict):
		res = resolver(**res)
	else:
		res = resolver(res)
	pkt = res.query(name, rr_type, rr_class, flags, tries)
	if pkt.rcode() == "SERVFAIL":
		raise Exception("%s lookup failed (server error or dnssec validation failed)" % name)
	if pkt.rcode() == "NXDOMAIN":
		if "AD" in pkt.flags():
			raise Exception("%s lookup failed (non-existence proven by DNSSEC)" % name )
		else:
			raise Exception("%s lookup failed" % name )
	if pkt.rcode() == "NOERROR":
		if "AD" not in pkt.flags():
			if not flex:
				raise Exception("DNS lookup was insecure")
			else:
				warnings.warn("DNS lookup was insecure")
		return pkt
	else:
		raise Exception("unknown ldns error, %s" % pkt.rcode())



class resolver:
	""" A wrapper around ldns.ldns_resolver. 

			**Examples**

			Making resolvers is easy!

			>>> from ldnsx import resolver
			>>> resolver() # from /etc/resolv.conf
			<resolver: 192.168.111.9>
			>>> resolver("") # resolver with no nameservers
			<resolver: >
			>>> resolver("193.110.157.135") #resolver pointing to ip addr
			<resolver: 193.110.157.135>
			>>> resolver("f.root-servers.net") # resolver pointing ip address(es) resolved from name
			<resolver: 2001:500:2f::f, 192.5.5.241>
			>>> resolver("193.110.157.135, 193.110.157.136") 
			>>> # resolver pointing to multiple ip addr, first takes precedence.
			<resolver: 193.110.157.136, 193.110.157.135>

			So is playing around with their nameservers!

			>>> import ldnsx
			>>> res = ldnsx.resolver("192.168.1.1")
			>>> res.add_nameserver("192.168.1.2")
			>>> res.add_nameserver("192.168.1.3")
			>>> res.nameservers_ip()
			["192.168.1.1","192.168.1.2","192.168.1.3"]

			And querying!

			>>> from ldnsx import resolver
			>>> res= resolver()
			>>> res.query("cow.com","A")
			;; ->>HEADER<<- opcode: QUERY, rcode: NOERROR, id: 7663
			;; flags: qr rd ra ; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 0 
			;; QUESTION SECTION:
			;; cow.com.     IN      A
			;; ANSWER SECTION:
			cow.com.        300     IN      A       208.87.34.18
			;; AUTHORITY SECTION:
			;; ADDITIONAL SECTION:
			;; Query time: 313 msec
			;; SERVER: 192.168.111.9
			;; WHEN: Fri Jun  3 11:01:02 2011
			;; MSG SIZE  rcvd: 41

	
			"""
	
	def __init__(self, ns = None, dnssec = False, tcp = False, port = 53):
		"""resolver constructor
			
			* ns    --  the nameserver/comma delimited nameserver list
			            defaults to settings from /etc/resolv.conf
			* dnssec -- should the resolver try and use dnssec or not?
		    * tcp -- should the resolver use TCP
		             'auto' is a depricated work around for old ldns problems
		    * port -- the port to use, must be the same for all nameservers

			"""
		# We construct based on a file and dump the nameservers rather than using
		# ldns_resolver_new() to avoid environment/configuration/magic specific 
		# bugs.
		self._ldns_resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
		if ns != None:
			self.drop_nameservers()
			nm_list = ns.split(',')
			nm_list = map(lambda s: s.strip(), nm_list)
			nm_list = filter(lambda s: s != "", nm_list)
			nm_list.reverse()
			for nm in nm_list:
				self.add_nameserver(nm)
		# Configure DNSSEC, tcp and port
		self.set_dnssec(dnssec)
		if tcp == 'auto':
			self.autotcp = True
			self._ldns_resolver.set_usevc(False)
		else:
			self.autotcp = False
			self._ldns_resolver.set_usevc(tcp)
		self._ldns_resolver.set_port(port)

	
	def query(self, name, rr_type, rr_class="IN", flags=["RD"], tries = 3):
		"""Run a query on the resolver.
				
			* name -- name to query for
			* rr_type -- the record type to query for
			* rr_class -- the class to query for, defaults to IN (Internet)
			* flags -- the flags to send the query with 
			* tries -- the number of times to attempt to acheive query in case of packet loss, etc
			
			**Examples**
			
			Let's get some A records!

			>>> google_a_records = resolver.query("google.com","A").answer()
			
			Using DNSSEC is easy :)

			>>> dnssec_pkt = ldnsx.resolver(dnssec=True).query("xelerance.com")
			
			We let you use strings to make things easy, but if you prefer stay close to DNS...

			>>> AAAA = 28
			>>> resolver.query("ipv6.google.com", AAAA)
			
			**More about rr_type**
			
			rr_type must be a supported resource record type. There are a large number of RR types:

			===========  ===================================  ==================
			TYPE         Value and meaning                    Reference
			===========  ===================================  ==================
			A            1 a host address                     [RFC1035]
			NS           2 an authoritative name server       [RFC1035]
			...
			AAAA         28 IP6 Address                       [RFC3596]
			...
			DS           43 Delegation Signer                 [RFC4034][RFC3658]
			...
			DNSKEY       48 DNSKEY                            [RFC4034][RFC3755]
			...
			Unassigned   32770-65279  
			Private use  65280-65534
			Reserved     65535 
			===========  ===================================  ==================
			
			(From http://www.iana.org/assignments/dns-parameters)

			RR types are given as a string (eg. "A"). In the case of Unassigned/Private use/Reserved ones,
			they are given as "TYPEXXXXX" where XXXXX is the number. ie. RR type 65280 is "TYPE65280". You 
			may also pass the integer, but you always be given the string.

			If the version of ldnsx you are using is old, it is possible that there could be new rr_types that
			we don't recognise mnemonic for. You can still use the number XXX or the string "TYPEXXX". To
			determine what rr_type menmonics we support, please refer to resolver.supported_rr_types()

		"""
		# Determine rr_type int
		if rr_type in _rr_types.keys():
			_rr_type = _rr_types[rr_type]
		elif isinstance(rr_type,int):
			_rr_type = rr_type
		elif isinstance(rr_type,str) and rr_type[0:4] == "TYPE":
			try:
				_rr_type = int(rr_type[4:])
			except:
				raise Exception("%s is a bad RR type. TYPEXXXX: XXXX must be a number")
		else:
			raise Exception("ldnsx (version %s) does not support the RR type %s."  % (__version__, str(rr_type)) ) 
		# Determine rr_class int
		if   rr_class == "IN": _rr_class = ldns.LDNS_RR_CLASS_IN 
		elif rr_class == "CH": _rr_class = ldns.LDNS_RR_CLASS_CH
		elif rr_class == "HS": _rr_class = ldns.LDNS_RR_CLASS_HS
		else:
			raise Exception("ldnsx (version %s) does not support the RR class %s." % (__version__, str(rr_class)) ) 
		# Determine flags int
		_flags = 0
		if "QR" in flags:  _flags |= ldns.LDNS_QR
		if "AA" in flags:  _flags |= ldns.LDNS_AA
		if "TC" in flags:  _flags |= ldns.LDNS_TC
		if "RD" in flags:  _flags |= ldns.LDNS_RD
		if "CD" in flags:  _flags |= ldns.LDNS_CD
		if "RA" in flags:  _flags |= ldns.LDNS_RA
		if "AD" in flags:  _flags |= ldns.LDNS_AD
		# Query
		if tries == 0: return None
		try:
			pkt = self._ldns_resolver.query(name, _rr_type, _rr_class, _flags)
		except KeyboardInterrupt: #Since so much time is spent waiting on ldns, this is very common place for Ctr-C to fall
			raise
		except: #Since the ldns exceptiion is not very descriptive...
			raise Exception("ldns backend ran into problems. Likely, the name you were querying for, %s, was invalid." % name)
		#Deal with failed queries
		if not pkt:
			if tries <= 1:
				return None
			else:
				# One of the major causes of none-packets is truncation of packets
				# When autotcp is set, we are in a flexible enough position to try and use tcp
				# to get around this.
				# Either way, we want to replace the resolver, since resolvers will sometimes
				# just freeze up.
				if self.autotcp:
					self = resolver( ",".join(self.nameservers_ip()),tcp=True, dnssec = self._ldns_resolver.dnssec())
					self.autotcp = True
					pkt = self.query(name, rr_type, rr_class=rr_class, flags=flags, tries = tries-1) 
					self._ldns_resolver.set_usevc(False)
					return pkt
				else:
					self = resolver( ",".join(self.nameservers_ip()), tcp = self._ldns_resolver.usevc(), dnssec = self._ldns_resolver.dnssec() )
					time.sleep(1) # It could be that things are failing because of a brief outage
					return self.query(name, rr_type, rr_class=rr_class, flags=flags, tries = tries-1) 
		elif self.autotcp:
			pkt = packet(pkt)
			if "TC" in pkt.flags():
				self._ldns_resolver.set_usevc(True)
				pkt2 = self.query(name, rr_type, rr_class=rr_class, flags=flags, tries = tries-1) 
				self._ldns_resolver.set_usevc(False)
				if pkt2: return packet(pkt2)
			return pkt
		return packet(pkt)
		#ret = []
		#for rr in pkt.answer().rrs():
		#	ret.append([str(rr.owner()),rr.ttl(),rr.get_class_str(),rr.get_type_str()]+[str(rdf) for rdf in rr.rdfs()])
		#return ret

	def suported_rr_types(self):
		""" Returns the supported DNS resource record types.

			Refer to resolver.query() for thorough documentation of resource 
			record types or refer to:

			http://www.iana.org/assignments/dns-parameters

		"""
		return _rr_types.keys()
	
	def AXFR(self,name):
		"""AXFR for name

			* name -- name to AXFR for
			
			This function is a generator. As it AXFRs it will yield you the records.

			**Example**

			Let's get a list of the tlds (gotta catch em all!):

			>>> tlds = []
			>>> for rr in resolver("f.root-servers.net").AXFR("."):
			>>>    if rr.rr_type() == "NS":
			>>>       tlds.append(rr.owner())

		"""
		#Dname seems to be unecessary on some computers, but it is on others. Avoid bugs.
		if self._ldns_resolver.axfr_start(ldns.ldns_dname(name), ldns.LDNS_RR_CLASS_IN) != ldns.LDNS_STATUS_OK:
			raise Exception("Starting AXFR failed. Error: %s" % ldns.ldns_get_errorstr_by_id(status))
		pres = self._ldns_resolver.axfr_next()
		while pres:
			yield resource_record(pres)
			pres = self._ldns_resolver.axfr_next()

	def nameservers_ip(self):
		""" returns a list of the resolvers nameservers (as IP addr)
		
		"""
		nm_stack2 =[]
		nm_str_stack2=[]
		nm = self._ldns_resolver.pop_nameserver()
		while nm:
			nm_stack2.append(nm)
			nm_str_stack2.append(str(nm))
			nm = self._ldns_resolver.pop_nameserver()
		for nm in nm_stack2:
			self._ldns_resolver.push_nameserver(nm)
		nm_str_stack2.reverse()
		return nm_str_stack2


	def add_nameserver(self,ns):
		""" Add a nameserver, IPv4/IPv6/name.

		"""
		if isValidIP(ns) == 4:
			address = ldns.ldns_rdf_new_frm_str(ldns.LDNS_RDF_TYPE_A,ns)
			self._ldns_resolver.push_nameserver(address)
		elif isValidIP(ns) == 6:
			address = ldns.ldns_rdf_new_frm_str(ldns.LDNS_RDF_TYPE_AAAA,ns)
			self._ldns_resolver.push_nameserver(address)
		else:
			resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
			#address = resolver.get_addr_by_name(ns)
			address = resolver.get_addr_by_name(ldns.ldns_dname(ns))
			if not address:
				address = resolver.get_addr_by_name(ldns.ldns_dname(ns))
				if not address:
					raise Exception("Failed to resolve address for %s" % ns)
			for rr in address.rrs():
				self._ldns_resolver.push_nameserver_rr(rr)

	def drop_nameservers(self):
		"""Drops all nameservers.
			This function causes the resolver to forget all nameservers.

		"""
		while self._ldns_resolver.pop_nameserver():
			pass

	def set_nameservers(self, nm_list):
		"""Takes a list of nameservers and sets the resolver to use them
		
		"""
		self.drop_nameservers()
		for nm in nm_list:
			self.add_nameserver(nm)

	def __repr__(self):
		return "<resolver: %s>" % ", ".join(self.nameservers_ip())
	__str__ = __repr__

	def set_dnssec(self,new_dnssec_status):
		"""Set whether the resolver uses DNSSEC.
		
		"""
		self._ldns_resolver.set_dnssec(new_dnssec_status)

class packet:
	
	def _construct_rr_filter(self, **kwds):
		def match(pattern, target):
			if pattern[0] in ["<",">","!"]:
				rel = pattern[0]
				pattern=pattern[1:]
			elif pattern[0:2] in ["<=","=>"]:
				rel = pattern[0:2]
				pattern=pattern[2:]
			else:
				rel = "="
			for val in pattern.split("|"):
				if {"<" : target <  val,
				    ">" : target >  val,
				    "!" : target != val,
				    "=" : target == val,
				    ">=": target >= val,
				    "<=": target <= val}[rel]:
					return True
			return False
		def f(rr):
			for key in kwds.keys():
				if ( ( isinstance(kwds[key], list) and str(rr[key]) not in map(str,kwds[key]) )
				  or ( not isinstance(kwds[key], list) and not match(str(kwds[key]), str(rr[key])))):
					return False
			return True
		return f
	
	def __init__(self, pkt):
		self._ldns_pkt = pkt
	
	def __repr__(self):
		return str(self._ldns_pkt)
	__str__ = __repr__
	
	def rcode(self):
		"""Returns the rcode.

		Example returned value: "NOERROR"

		possilbe rcodes (via ldns): "FORMERR", "MASK", "NOERROR",
		"NOTAUTH", "NOTIMPL", "NOTZONE", "NXDOMAIN",
		"NXRSET", "REFUSED", "SERVFAIL", "SHIFT", 
		"YXDOMAIN", "YXRRSET"

		Refer to http://www.iana.org/assignments/dns-parameters
		section: DNS RCODEs
		"""
		return self._ldns_pkt.rcode2str()

	def opcode(self):
		"""Returns the rcode.

		Example returned value: "QUERY"

		"""
		return self._ldns_pkt.opcode2str()
	
	def flags(self):
		"""Return packet flags (as list of strings).
		
		Example returned value: ['QR', 'RA', 'RD']

		**What are the flags?**
		
		========  ====  =====================  =========
		Bit       Flag  Description            Reference
		========  ====  =====================  =========
		bit 5     AA    Authoritative Answer   [RFC1035]
		bit 6     TC    Truncated Response     [RFC1035]
		bit 7     RD    Recursion Desired      [RFC1035]
		bit 8     RA    Recursion Allowed      [RFC1035]
		bit 9           Reserved
		bit 10    AD    Authentic Data         [RFC4035]
		bit 11    CD    Checking Disabled      [RFC4035]
		========  ====  =====================  =========

		(from http://www.iana.org/assignments/dns-parameters)

		There is also QR. It is mentioned in other sources,
		though not the above page. It being false means that
		the packet is a query, it being true means that it is
		a response.

		"""
		ret = []
		if self._ldns_pkt.aa(): ret += ["AA"]
		if self._ldns_pkt.ad(): ret += ["AD"]
		if self._ldns_pkt.cd(): ret += ["CD"]
		if self._ldns_pkt.qr(): ret += ["QR"]
		if self._ldns_pkt.ra(): ret += ["RA"]
		if self._ldns_pkt.rd(): ret += ["RD"]
		if self._ldns_pkt.tc(): ret += ["TC"]
		return ret

	def answer(self, **filters):
		"""Returns the answer section.
		
		* filters -- a filtering mechanism
		
		Since a very common desire is to filter the resource records in a packet
		section, we provide a special tool for doing this: filters. They are a
		lot like regular python filters, but more convenient. If you set a 
		field equal to some value, you will only receive resource records for which
		it holds true.

		**Examples**

		>>> res = ldnsx.resolver()
		>>> pkt = res.query("google.ca","A")
		>>> pkt.answer()
		[google.ca.     28      IN      A       74.125.91.99
		, google.ca.    28      IN      A       74.125.91.105
		, google.ca.    28      IN      A       74.125.91.147
		, google.ca.    28      IN      A       74.125.91.103
		, google.ca.    28      IN      A       74.125.91.104
		, google.ca.    28      IN      A       74.125.91.106
		]

		To understand filters, consider the following:

		>>> pkt = ldnsx.query("cow.com","ANY")
		>>> pkt.answer()
		[cow.com.       276     IN      A       208.87.32.75
		, cow.com.      3576    IN      NS      sell.internettraffic.com.
		, cow.com.      3576    IN      NS      buy.internettraffic.com.
		, cow.com.      3576    IN      SOA     buy.internettraffic.com. hostmaster.hostingnet.com. 1308785320 10800 3600 604800 3600
		]
		>>> pkt.answer(rr_type="A")
		[cow.com.       276     IN      A       208.87.32.75
		]
		>>> pkt.answer(rr_type="A|NS")
		[cow.com.       276     IN      A       208.87.32.75
		, cow.com.      3576    IN      NS      sell.internettraffic.com.
		, cow.com.      3576    IN      NS      buy.internettraffic.com.
		]
		>>> pkt.answer(rr_type="!NS")
		[cow.com.       276     IN      A       208.87.32.75
		, cow.com.      3576    IN      SOA     buy.internettraffic.com. hostmaster.hostingnet.com. 1308785320 10800 3600 604800 3600
		]
		
		fields are the same as when indexing a resource record.
		note: ordering is alphabetical.
		"""
		ret =  [resource_record(rr) for rr in self._ldns_pkt.answer().rrs()]
		return filter(self._construct_rr_filter(**filters), ret)

	def authority(self, **filters):
		"""Returns the authority section.

		* filters -- a filtering mechanism
		
		Since a very common desire is to filter the resource records in a packet
		section, we provide a special tool for doing this: filters. They are a
		lot like regular python filters, but more convenient. If you set a 
		field equal to some value, you will only receive resource records for which
		it holds true. See answer() for details.

		**Examples**

		>>> res = ldnsx.resolver()
		>>> pkt = res.query("google.ca","A")
		>>> pkt.authority()
		[google.ca.     251090  IN      NS      ns3.google.com.
		, google.ca.    251090  IN      NS      ns1.google.com.
		, google.ca.    251090  IN      NS      ns2.google.com.
		, google.ca.    251090  IN      NS      ns4.google.com.
		]

		"""
		ret = [resource_record(rr) for rr in self._ldns_pkt.authority().rrs()]
		return filter(self._construct_rr_filter(**filters), ret)

	def additional(self, **filters):
		"""Returns the additional section.

		* filters -- a filtering mechanism
		
		Since a very common desire is to filter the resource records in a packet
		section, we provide a special tool for doing this: filters. They are a
		lot like regular python filters, but more convenient. If you set a 
		field equal to some value, you will only receive resource records for which
		it holds true. See answer() for details.

		**Examples**

		>>> res = ldnsx.resolver()
		>>> pkt = res.query("google.ca","A")
		>>> pkt.additional()
		[ns3.google.com.        268778  IN      A       216.239.36.10
		, ns1.google.com.       262925  IN      A       216.239.32.10
		, ns2.google.com.       255659  IN      A       216.239.34.10
		, ns4.google.com.       264489  IN      A       216.239.38.10
		]

		"""
		ret = [resource_record(rr) for rr in self._ldns_pkt.additional().rrs()]
		return filter(self._construct_rr_filter(**filters), ret)

	def question(self, **filters):
		"""Returns the question section.

		* filters -- a filtering mechanism
		
		Since a very common desire is to filter the resource records in a packet
		section, we provide a special tool for doing this: filters. They are a
		lot like regular python filters, but more convenient. If you set a 
		field equal to some value, you will only receive resource records for which
		it holds true. See answer() for details.

		"""
		ret = [resource_record(rr) for rr in self._ldns_pkt.question().rrs()]
		return filter(self._construct_rr_filter(**filters), ret)

class resource_record:
	
	_rdfs = None
	_iter_pos = None
	
	def __init__(self, rr):
		self._ldns_rr = rr
		self._rdfs = [str(rr.owner()),rr.ttl(),rr.get_class_str(),rr.get_type_str()]+[str(rdf) for rdf in rr.rdfs()]
	
	def __repr__(self):
		return str(self._ldns_rr)
	
	__str__ = __repr__

	def __iter__(self):
		self._iter_pos = 0
		return self

	def next(self):
		if self._iter_pos < len(self._rdfs):
			self._iter_pos += 1
			return self._rdfs[self._iter_pos-1]
		else:
			raise StopIteration

	def __len__(self):
		try:
			return len(self._rdfs)
		except:
			return 0

	def __getitem__(self, n):
		if isinstance(n, int):
			return self._rdfs[n]
		elif isinstance(n, str):
			n = n.lower()
			if n in ["owner"]:
				return self.owner()
			elif n in ["rr_type", "rr type", "type"]:
				return self.rr_type()
			elif n in ["rr_class", "rr class", "class"]:
				return self.rr_class()
			elif n in ["covered_type", "covered type", "type2"]:
				return self.covered_type()
			elif n in ["ttl"]:
				return self.ttl()
			elif n in ["ip"]:
				return self.ip()			
			elif n in ["alg", "algorithm"]:
				return self.alg()
			elif n in ["protocol"]:
				return self.protocol()
			elif n in ["flags"]:
				return self.flags()
			else:
				raise Exception("ldnsx (version %s) does not recognize the rr field %s" % (__version__,n) ) 
		else:
			raise TypeError("bad type %s for index resource record" % type(n) ) 
			

	#def rdfs(self):
	#	return self._rdfs.clone()
	
	def owner(self):
		"""Get the RR's owner"""
		return str(self._ldns_rr.owner())
	
	def rr_type(self):
		"""Get a RR's type """
		return self._ldns_rr.get_type_str()

	def covered_type(self):
		"""Get an RRSIG RR's covered type"""
		if self.rr_type() == "RRSIG":
			return self[4]
		else:
			return ""
	
	def rr_class(self):
		"""Get the RR's collapse"""
		return self._ldns_rr.get_class_str()
	
	def ttl(self):
		"""Get the RR's TTL"""
		return self._ldns_rr.ttl()

	def inception(self, out_format="UTC"):
		"""returns the inception time in format out_format, defaulting to a UTC string. 
		options for out_format are:

		   UTC -- a UTC string eg. 20110712192610 (2011/07/12 19:26:10) 
		   unix -- number of seconds since the epoch, Jan 1, 1970
		   struct_time -- the format used by python's time library
		"""
		# Something very strange is going on with inception/expiration dates in DNS.
		# According to RFC 4034 section 3.1.5 (http://tools.ietf.org/html/rfc4034#page-9)
		# the inception/expiration fields should be in seconds since Jan 1, 1970, the Unix
		# epoch (as is standard in unix). Yet all the packets I've seen provide UTC encoded
		# as a string instead, eg. "20110712192610" which is 2011/07/12 19:26:10. 
		#
		# It turns out that this is a standard thing that ldns is doing before the data gets 
		# to us.
		if self.rr_type() == "RRSIG":
			if out_format.lower() in ["utc", "utc str", "utc_str"]:
				return self[9]
			elif out_format.lower() in ["unix", "posix", "ctime"]:
				return calendar.timegm(time.strptime(self[9], "%Y%m%d%H%M%S"))
			elif out_format.lower() in ["relative"]:
				return calendar.timegm(time.strptime(self[9], "%Y%m%d%H%M%S")) - time.time()
			elif out_format.lower() in ["struct_time", "time.struct_time"]:
				return time.strptime(self[9], "%Y%m%d%H%M%S")
			else:
				raise Exception("unrecognized time format")
		else:
			return ""

	def expiration(self, out_format="UTC"):
		"""get expiration time. see inception() for more information"""
		if self.rr_type() == "RRSIG":
			if out_format.lower() in ["utc", "utc str", "utc_str"]:
				return self[8]
			elif out_format.lower() in ["unix", "posix", "ctime"]:
				return calendar.timegm(time.strptime(self[8], "%Y%m%d%H%M%S"))
			elif out_format.lower() in ["relative"]:
				return calendar.timegm(time.strptime(self[8], "%Y%m%d%H%M%S")) - time.time()
			elif out_format.lower() in ["struct_time", "time.struct_time"]:
				return time.strptime(self[8], "%Y%m%d%H%M%S")
			else:
				raise Exception("unrecognized time format")
		else:
			return ""

	def ip(self):
		""" IP address form A/AAAA record"""
		if self.rr_type() in ["A", "AAAA"]:
			return self[4]
		else:
			raise Exception("ldnsx does not support ip for records other than A/AAAA")

	def alg(self):
		"""Returns algorithm of RRSIG/DNSKEY/DS"""
		t = self.rr_type() 
		if t == "RRSIG":
			return int(self[5])
		elif t == "DNSKEY":
			return int(self[6])
		elif t == "DS":
			return int(self[5])
		else:
			return -1

	def protocol(self):
		""" Returns proticol of the DNSKEY"""
		t = self.rr_type() 
		if t == "DNSKEY":
			return int(self[5])
		else:
			return -1

	def flags(self):
		"""Return RR flags for DNSKEY """
		t = self.rr_type() 
		if t == "DNSKEY":
			ret = []
			n = int(self[4])
			for m in range(1):
				if 2**(15-m) & n:
					if   m == 7: ret.append("ZONE")
					elif m == 8: ret.append("REVOKE")
					elif m ==15: ret.append("SEP")
					else:        ret.append(m)
			return ret
		else:
			return []

_rr_types={
	"A"    : ldns.LDNS_RR_TYPE_A,
	"A6"   : ldns.LDNS_RR_TYPE_A6,
	"AAAA" : ldns.LDNS_RR_TYPE_AAAA,
	"AFSDB": ldns.LDNS_RR_TYPE_AFSDB,
	"ANY"  : ldns.LDNS_RR_TYPE_ANY,
	"APL"  : ldns.LDNS_RR_TYPE_APL,
	"ATMA" : ldns.LDNS_RR_TYPE_ATMA,
	"AXFR" : ldns.LDNS_RR_TYPE_AXFR,
	"CERT" : ldns.LDNS_RR_TYPE_CERT,
	"CNAME": ldns.LDNS_RR_TYPE_CNAME,
	"COUNT": ldns.LDNS_RR_TYPE_COUNT,
	"DHCID": ldns.LDNS_RR_TYPE_DHCID,
	"DLV"  : ldns.LDNS_RR_TYPE_DLV,
	"DNAME": ldns.LDNS_RR_TYPE_DNAME,
	"DNSKEY": ldns.LDNS_RR_TYPE_DNSKEY,
	"DS"   : ldns.LDNS_RR_TYPE_DS,
	"EID"  : ldns.LDNS_RR_TYPE_EID,
	"FIRST": ldns.LDNS_RR_TYPE_FIRST,
	"GID"  : ldns.LDNS_RR_TYPE_GID,
	"GPOS" : ldns.LDNS_RR_TYPE_GPOS,
	"HINFO": ldns.LDNS_RR_TYPE_HINFO,
	"IPSECKEY": ldns.LDNS_RR_TYPE_IPSECKEY,
	"ISDN" : ldns.LDNS_RR_TYPE_ISDN,
	"IXFR" : ldns.LDNS_RR_TYPE_IXFR,
	"KEY"  : ldns.LDNS_RR_TYPE_KEY,
	"KX"   : ldns.LDNS_RR_TYPE_KX,
	"LAST" : ldns.LDNS_RR_TYPE_LAST,
	"LOC"  : ldns.LDNS_RR_TYPE_LOC,
	"MAILA": ldns.LDNS_RR_TYPE_MAILA,
	"MAILB": ldns.LDNS_RR_TYPE_MAILB,
	"MB"   : ldns.LDNS_RR_TYPE_MB,
	"MD"   : ldns.LDNS_RR_TYPE_MD,
	"MF"   : ldns.LDNS_RR_TYPE_MF,
	"MG"   : ldns.LDNS_RR_TYPE_MG,
	"MINFO": ldns.LDNS_RR_TYPE_MINFO,
	"MR"   : ldns.LDNS_RR_TYPE_MR,
	"MX"   : ldns.LDNS_RR_TYPE_MX,
	"NAPTR": ldns.LDNS_RR_TYPE_NAPTR,
	"NIMLOC": ldns.LDNS_RR_TYPE_NIMLOC,
	"NS"   : ldns.LDNS_RR_TYPE_NS,
	"NSAP" : ldns.LDNS_RR_TYPE_NSAP,
	"NSAP_PTR" : ldns.LDNS_RR_TYPE_NSAP_PTR,
	"NSEC" : ldns.LDNS_RR_TYPE_NSEC,
	"NSEC3": ldns.LDNS_RR_TYPE_NSEC3,
	"NSEC3PARAM" : ldns.LDNS_RR_TYPE_NSEC3PARAM,
	"NSEC3PARAMS" : ldns.LDNS_RR_TYPE_NSEC3PARAMS,
	"NULL" : ldns.LDNS_RR_TYPE_NULL,
	"NXT"  : ldns.LDNS_RR_TYPE_NXT,
	"OPT"  : ldns.LDNS_RR_TYPE_OPT,
	"PTR"  : ldns.LDNS_RR_TYPE_PTR,
	"PX"   : ldns.LDNS_RR_TYPE_PX,
	"RP"   : ldns.LDNS_RR_TYPE_RP,
	"RRSIG": ldns.LDNS_RR_TYPE_RRSIG,
	"RT"   : ldns.LDNS_RR_TYPE_RT,
	"SIG"  : ldns.LDNS_RR_TYPE_SIG,
	"SINK" : ldns.LDNS_RR_TYPE_SINK,
	"SOA"  : ldns.LDNS_RR_TYPE_SOA,
	"SRV"  : ldns.LDNS_RR_TYPE_SRV,
	"SSHFP": ldns.LDNS_RR_TYPE_SSHFP,
	"TSIG" : ldns.LDNS_RR_TYPE_TSIG,
	"TXT"  : ldns.LDNS_RR_TYPE_TXT,
	"UID"  : ldns.LDNS_RR_TYPE_UID,
	"UINFO": ldns.LDNS_RR_TYPE_UINFO,
	"UNSPEC": ldns.LDNS_RR_TYPE_UNSPEC,
	"WKS"  : ldns.LDNS_RR_TYPE_WKS,
	"X25"  : ldns.LDNS_RR_TYPE_X25
}




#!/usr/bin/python
#
# MX is a small program that prints out the mx records for a particular domain
#
import ldns

dname = ldns.ldns_dname("nic.cz")
print(dname)

resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")

pkt = resolver.query(dname, ldns.LDNS_RR_TYPE_MX,ldns.LDNS_RR_CLASS_IN)

if (pkt):
    mx = pkt.rr_list_by_type(ldns.LDNS_RR_TYPE_MX, ldns.LDNS_SECTION_ANSWER)
    if (mx):
       mx.sort()
       print(mx)



import ldns
import sys

if len(sys.argv) <= 1:
    print "Usage: %s zone_file" % sys.argv[0]
    sys.exit()

inp = open(sys.argv[1],"r");
for rr in ldns.ldns_rr_iter_frm_fp_l(inp):
  print rr

inp.close()

    




#!/usr/bin/env python

#
# ldns_resolver testing script.
#
# Do not use constructs that differ between Python 2 and 3.
# Use write on stdout or stderr.
#


import ldns
import sys
import os
import inspect


class_name = "ldns_resolver"
method_name = None
error_detected = False
temp_fname = "tmp_resolver.txt"


def set_error():
    """
        Writes an error message and sets error flag.
    """
    global class_name
    global method_name
    global error_detected
    error_detected = True
    sys.stderr.write("(line %d): malfunctioning method %s.\n" % \
       (inspect.currentframe().f_back.f_lineno, method_name))


#if not error_detected:
if True:
    method_name = class_name + ".axfr_complete()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".axfr_last_pkt()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".axfr_next()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".axfr_start()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".debug()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_debug(False)
    try:
        ret = resolver.debug()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_debug(True)
    try:
        ret = resolver.debug()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".dec_nameserver_count()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    cnt = resolver.nameserver_count()
    try:
        resolver.dec_nameserver_count()
    except:
        set_error()
    if cnt != (resolver.nameserver_count() + 1):
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".defnames()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_defnames(False)
    try:
        ret = resolver.defnames()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_defnames(True)
    try:
        ret = resolver.defnames()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".dnsrch()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_dnsrch(False)
    try:
        ret = resolver.dnsrch()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_dnsrch(True)
    try:
        ret = resolver.dnsrch()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".dnssec()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_dnssec(False)
    try:
        ret = resolver.dnssec()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_dnssec(True)
    try:
        ret = resolver.dnssec()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".dnssec_anchors()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    rrl = ldns.ldns_rr_list.new()
    try:
        ret = resolver.dnssec_anchors()
        if ret != None:
            set_error()
    except:
        set_error()
    resolver.set_dnssec_anchors(rrl)
    try:
        ret = resolver.dnssec_anchors()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".dnssec_cd()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_dnssec_cd(False)
    try:
        ret = resolver.dnssec_cd()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_dnssec_cd(True)
    try:
        ret = resolver.dnssec_cd()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".domain()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_domain(None)
    try:
        ret = resolver.domain()
        if ret != None:
            set_error()
    except:
        set_error()
    dname = ldns.ldns_dname("example.com.")
    resolver.set_domain(dname)
    try:
        ret = resolver.domain()
        if not isinstance(ret, ldns.ldns_dname):
            set_error()
        if ret != dname:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".edns_udp_size()"
    try:
        resolver = ldns.ldns_resolver.new()
        if not isinstance(resolver, ldns.ldns_resolver):
            set_error()
    except:
        set_error()



#if not error_detected:
if True:
    method_name = class_name + ".edns_udp_size()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_edns_udp_size(4096)
    try:
        ret = resolver.edns_udp_size()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 4096:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".fail()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_fail(False)
    try:
        ret = resolver.fail()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_fail(True)
    try:
        ret = resolver.fail()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".fallback()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_fallback(False)
    try:
        ret = resolver.fallback()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_fallback(True)
    try:
        ret = resolver.fallback()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".get_addr_by_name()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.get_addr_by_name("www.google.com", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        ret = resolver.get_addr_by_name(1, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.get_addr_by_name("www.google.com", "bad argument", ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.get_addr_by_name("www.google.com", ldns.LDNS_RR_CLASS_IN, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".get_name_by_addr()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        addr = resolver.get_name_by_addr("8.8.8.8", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        if not isinstance(addr, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        addr = resolver.get_name_by_addr(1, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        addr = resolver.get_name_by_addr("8.8.8.8", "bad argument", ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        addr = resolver.get_name_by_addr("8.8.8.8", ldns.LDNS_RR_CLASS_IN, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()



#if not error_detected:
if True:
    method_name = class_name + ".igntc()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_igntc(False)
    try:
        ret = resolver.igntc()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_igntc(True)
    try:
        ret = resolver.igntc()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".incr_nameserver_count()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    cnt = resolver.nameserver_count()
    try:
        resolver.incr_nameserver_count()
    except:
        set_error()
    if (cnt + 1) != resolver.nameserver_count():
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".ip6()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_ip6(0)
    try:
        ret = resolver.ip6()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".nameserver_count()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_nameserver_count(1)
    try:
        ret = resolver.nameserver_count()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 1:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".nameserver_rtt()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    num = resolver.nameserver_count()
    for i in range(0, num):
        resolver.set_nameserver_rtt(i, i + 1)
    try:
        for i in range(0, num):
            ret = resolver.nameserver_rtt(i)
            if (not isinstance(ret, int)) and (not isinstance(ret, long)):
                set_error()
            if (i + 1) != ret:
                set_error()
    except:
        set_error()
    try:
        ret = resolver.nameserver_rtt("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".nameservers()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".nameservers_randomize()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.nameservers_randomize()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".new_frm_file()"
    try:
        ret = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf", raiseException=True)
        if not isinstance(ret, ldns.ldns_resolver):
            set_error()
    except:
        set_error()
    try:
        ret = ldns.ldns_resolver.new_frm_file(1, raiseException=True)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".new_frm_fp()"
    fi = open("/etc/resolv.conf")
    try:
        ret = ldns.ldns_resolver.new_frm_fp(fi, raiseException=True)
        if not isinstance(ret, ldns.ldns_resolver):
            set_error()
    except:
        set_error()
    fi.close()
    try:
        ret = ldns.ldns_resolver.new_frm_fp(1, raiseException=True)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".new_frm_fp_l()"
    fi = open("/etc/resolv.conf")
    try:
        ret, line = ldns.ldns_resolver.new_frm_fp_l(fi, raiseException=True)
        if not isinstance(ret, ldns.ldns_resolver):
            set_error()
        if (not isinstance(line, int)) and (not isinstance(line, long)):
            set_error()
    except:
        set_error()
    fi.close()
    try:
        ret, line = ldns.ldns_resolver.new_frm_fp_l(1, raiseException=True)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".pop_nameserver()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    cnt = resolver.nameserver_count()
    try:
        for i in range(0, cnt):
            ret = resolver.pop_nameserver()
            if not isinstance(ret, ldns.ldns_rdf):
                set_error()
    except:
        set_error()
    try:
        ret = resolver.pop_nameserver()
        if ret != None:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".port()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_port(12345)
    try:
        ret = resolver.port()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 12345:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".prepare_query_pkt()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.prepare_query_pkt("example.com.", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD, raiseException=True)
        if not isinstance(ret, ldns.ldns_pkt):
            set_error()
    except:
        set_error()
    try:
        ret = resolver.prepare_query_pkt(1, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD, raiseException=True)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.prepare_query_pkt("example.com.", "bad argument", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD, raiseException=True)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.prepare_query_pkt("example.com.", ldns.LDNS_RR_TYPE_A, "bad argument", ldns.LDNS_RD, raiseException=True)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.prepare_query_pkt("example.com.", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, "bad argument", raiseException=True)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".push_dnssec_anchor()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    key = ldns.ldns_key.new_frm_algorithm(ldns.LDNS_SIGN_DSA, 512)
    domain = ldns.ldns_dname("example.")
    key.set_pubkey_owner(domain)
    pubkey = key.key_to_rr()
    ds = ldns.ldns_key_rr2ds(pubkey, ldns.LDNS_SHA1)
    try:
        ret = resolver.push_dnssec_anchor(ds)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    rr = ldns.ldns_rr.new_frm_str("test1 600 IN A 0.0.0.0")
    try:
        ret = resolver.push_dnssec_anchor(rr)
        if ret == ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    try:
        ret = resolver.push_dnssec_anchor("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".push_nameserver()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    rdf = ldns.ldns_rdf.new_frm_str("127.0.0.1", ldns.LDNS_RDF_TYPE_A)
    try:
        ret = resolver.push_nameserver(rdf)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    rdf = ldns.ldns_rdf.new_frm_str("::1", ldns.LDNS_RDF_TYPE_AAAA)
    try:
        ret = resolver.push_nameserver(rdf)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    rdf = ldns.ldns_rdf.new_frm_str("example.com.", ldns.LDNS_RDF_TYPE_DNAME)
    try:
        ret = resolver.push_nameserver(rdf)
        if ret == ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    try:
        ret = resolver.push_nameserver("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()

#if not error_detected:
if True:
    method_name = class_name + ".push_nameserver_rr()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    rr = ldns.ldns_rr.new_frm_str("test 600 IN A 127.0.0.1")
    try:
        ret = resolver.push_nameserver_rr(rr)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    rr = ldns.ldns_rr.new_frm_str("test 600 IN AAAA ::1")
    try:
        ret = resolver.push_nameserver_rr(rr)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    rr = ldns.ldns_rr.new_frm_str("test 600 IN NS 8.8.8.8")
    try:
        ret = resolver.push_nameserver_rr(rr)
        if ret == ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    try:
        ret = resolver.push_nameserver_rr("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".push_nameserver_rr_list()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test 600 IN A 127.0.0.1")
    rrl.push_rr(rr)
    try:
        ret = resolver.push_nameserver_rr_list(rrl)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test 600 IN AAAA ::1")
    rrl.push_rr(rr)
    try:
        ret = resolver.push_nameserver_rr_list(rrl)
        if ret != ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    rrl = ldns.ldns_rr_list.new()
    rr = ldns.ldns_rr.new_frm_str("test 600 IN NS 8.8.8.8")
    rrl.push_rr(rr)
    try:
        ret = resolver.push_nameserver_rr_list(rrl)
        if ret == ldns.LDNS_STATUS_OK:
            set_error()
    except:
        set_error()
    try:
        ret = resolver.push_nameserver_rr_list("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".push_searchlist()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.push_searchlist("example.com.")
    try:
        resolver.push_searchlist("example.com.")
    except:
        set_error()
    try:
        resolver.push_searchlist(1)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".query()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        if not isinstance(ret, ldns.ldns_pkt):
            set_error()
    except:
        set_error()
    try:
        ret = resolver.query(1, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.query("www.nic.cz", "bad argument", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, "bad argument", ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.query("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".random()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_random(False)
    try:
        ret = resolver.random()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_random(True)
    try:
        ret = resolver.random()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".recursive()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_recursive(False)
    try:
        ret = resolver.recursive()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_recursive(True)
    try:
        ret = resolver.recursive()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".retrans()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_retrans(127)
    try:
        ret = resolver.retrans()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 127:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".retry()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_retry(4)
    try:
        ret = resolver.retry()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 4:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".rtt()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".search()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.search("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        if not isinstance(ret, ldns.ldns_pkt):
            set_error()
    except:
        set_error()
    try:
        ret = resolver.search(1, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.search("www.nic.cz", "bad argument", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.search("www.nic.cz", ldns.LDNS_RR_TYPE_A, "bad argument", ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.search("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".searchlist()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".searchlist_count()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.searchlist_count()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 0:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".send()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.send("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        if not isinstance(ret, ldns.ldns_pkt):
            set_error()
    except:
        set_error()
    try:
        ret = resolver.send(1, ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.send("www.nic.cz", "bad argument", ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.send("www.nic.cz", ldns.LDNS_RR_TYPE_A, "bad argument", ldns.LDNS_RD)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.send("www.nic.cz", ldns.LDNS_RR_TYPE_A, ldns.LDNS_RR_CLASS_IN, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".send_pkt()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    pkt = ldns.ldns_pkt.new_query_frm_str("test.nic.cz",ldns.LDNS_RR_TYPE_ANY, ldns.LDNS_RR_CLASS_IN, ldns.LDNS_RD | ldns.LDNS_AD)
    try:
        status, ret = resolver.send_pkt(pkt)
        if status != ldns.LDNS_STATUS_OK:
            ste_error()
        if not isinstance(ret, ldns.ldns_pkt):
            set_error()
    except:
        set_error()
    try:
        status, ret = resolver.send_pkt("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_debug()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_debug(False)
        ret = resolver.debug()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_debug(True)
        ret = resolver.debug()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_defnames()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_defnames(False)
        ret = resolver.defnames()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_defnames(True)
        ret = resolver.defnames()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_dnsrch()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_dnsrch(False)
        ret = resolver.dnsrch()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_dnsrch(True)
        ret = resolver.dnsrch()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_dnssec()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_dnssec(False)
        ret = resolver.dnssec()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_dnssec(True)
        ret = resolver.dnssec()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_dnssec_anchors()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    rrl = ldns.ldns_rr_list.new()
    try:
        resolver.set_dnssec_anchors(rrl)
        ret = resolver.dnssec_anchors()
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
    except:
        set_error()
    try:
        resolver.set_dnssec_anchors("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_dnssec_cd()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_dnssec_cd(False)
        ret = resolver.dnssec_cd()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_dnssec_cd(True)
        ret = resolver.dnssec_cd()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_domain()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_domain(None)
        ret = resolver.domain()
        if ret != None:
            set_error()
    except:
        set_error()
    dname = ldns.ldns_dname("example.com.")
    try:
        resolver.set_domain(dname)
        ret = resolver.domain()
        if not isinstance(ret, ldns.ldns_dname):
            set_error()
        if ret != dname:
            set_error()
    except:
        set_error()
    rdf = ldns.ldns_rdf.new_frm_str("example.com.", ldns.LDNS_RDF_TYPE_DNAME)
    try:
        resolver.set_domain(rdf)
        ret = resolver.domain()
        if not isinstance(ret, ldns.ldns_rdf):
            set_error()
        if ret != dname:
            set_error()
    except:
        set_error()
    resolver.set_domain("example.com.")
    try:
        resolver.set_domain("example.com.")
        ret = resolver.domain()
        if not isinstance(ret, ldns.ldns_dname):
            set_error()
        if ret != dname:
            set_error()
    except:
        set_error()
    rdf = ldns.ldns_rdf.new_frm_str("127.0.0.1", ldns.LDNS_RDF_TYPE_A)
    try:
        resolver.set_domain(rdf)
        set_error()
    except Exception as e:
        pass
    except:
        set_error()
    try:
        resolver.set_domain(1)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_edns_udp_size()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_edns_udp_size(4096)
        ret = resolver.edns_udp_size()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 4096:
            set_error()
    except:
        set_error()
    try:
        resolver.set_edns_udp_size("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        ste_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_fail()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_fail(False)
        ret = resolver.fail()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_fail(True)
        ret = resolver.fail()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_fallback()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_fallback(False)
        ret = resolver.fallback()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_fallback(True)
        ret = resolver.fallback()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_igntc()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_igntc(False)
        ret = resolver.igntc()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_igntc(True)
        ret = resolver.igntc()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_ip6()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_ip6(1)
        ret = resolver.ip6()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 1:
            set_error()
    except:
        set_error()
    try:
        resolver.set_ip6("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        ste_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_nameserver_count()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_nameserver_count(2)
        ret = resolver.nameserver_count()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 2:
            set_error()
    except:
        set_error()
    try:
        resolver.set_nameserver_count("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_nameserver_rtt()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    num = resolver.nameserver_count()
    try:
        for i in range(0, num):
            resolver.set_nameserver_rtt(i, i + 1)
            ret = resolver.nameserver_rtt(i)
            if (not isinstance(ret, int)) and (not isinstance(ret, long)):
                set_error()
            if (i + 1) != ret:
                set_error()
    except:
        set_error()
    try:
        ret = resolver.set_nameserver_rtt("bad argument", 0)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()
    try:
        ret = resolver.set_nameserver_rtt(0, "bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_nameservers()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".set_port()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_port(12345)
        ret = resolver.port()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 12345:
            set_error()
    except:
        set_error()
    try:
        resolver.set_port("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_random()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_random(False)
        ret = resolver.random()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_random(True)
        ret = resolver.random()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_recursive()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_recursive(False)
        ret = resolver.recursive()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_recursive(True)
        ret = resolver.recursive()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_retrans()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_retrans(127)
        ret = resolver.retrans()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 127:
            set_error()
    except:
        set_error()
    try:
        resolver.set_retrans("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_retry()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_retry(4)
        ret = resolver.retry()
        if (not isinstance(ret, int)) and (not isinstance(ret, long)):
            set_error()
        if ret != 4:
            set_error()
    except:
        set_error()
    try:
        resolver.set_retry("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_rtt()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".set_timeout()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".set_tsig_algorithm()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    tsigstr = "hmac-md5.sig-alg.reg.int."
    try:
        resolver.set_tsig_algorithm(tsigstr)
        ret = resolver.tsig_algorithm()
        if not isinstance(ret, str):
            set_error()
        if ret != tsigstr:
            set_error()
    except:
        set_error()
    try:
        resolver.set_tsig_algorithm(1)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_tsig_keydata()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    tkdstr = "Humpty Dumpty sat on a wall, Humpty Dumpty had a great fall, All the King's horses and all the King's men, Couldn't put Humpty together again."
    try:
        resolver.set_tsig_keydata(tkdstr)
        ret = resolver.tsig_keydata()
        if not isinstance(ret, str):
            set_error()
        if ret != tkdstr:
            set_error()
    except:
        set_error()
    try:
        resolver.set_tsig_keydata(1)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_tsig_keyname()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    tknstr = "key 1"
    try:
        resolver.set_tsig_keyname(tknstr)
        ret = resolver.tsig_keyname()
        if not isinstance(ret, str):
            set_error()
        if ret != tknstr:
            set_error()
    except:
        set_error()
    try:
        resolver.set_tsig_keyname(1)
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".set_usevc()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        resolver.set_usevc(False)
        ret = resolver.usevc()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    try:
        resolver.set_usevc(True)
        ret = resolver.usevc()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".timeout()"
    sys.stderr.write("%s not tested.\n" % (method_name))


#if not error_detected:
if True:
    method_name = class_name + ".trusted_key()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    key = ldns.ldns_key.new_frm_algorithm(ldns.LDNS_SIGN_DSA, 512)
    domain = ldns.ldns_dname("example.")
    key.set_pubkey_owner(domain)
    pubkey = key.key_to_rr()
    ds = ldns.ldns_key_rr2ds(pubkey, ldns.LDNS_SHA1)
    resolver.push_dnssec_anchor(ds)
    rrl = ldns.ldns_rr_list.new()
    try:
        ret = resolver.trusted_key(rrl)
        if ret != None:
            set_error()
    except:
        set_error()
    rrl.push_rr(ds)
    ret = resolver.trusted_key(rrl)
    try:
        ret = resolver.trusted_key(rrl)
        if not isinstance(ret, ldns.ldns_rr_list):
            set_error()
        if ret.rr_count() != 1:
            set_error()
    except:
        set_error()
    try:
        ret = resolver.trusted_key("bad argument")
        set_error()
    except TypeError as e:
        pass
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".tsig_algorithm()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.tsig_algorithm()
        if ret != None:
            set_error()
    except:
        set_error()
    tsigstr = "hmac-md5.sig-alg.reg.int."
    resolver.set_tsig_algorithm(tsigstr)
    try:
        ret = resolver.tsig_algorithm()
        if not isinstance(ret, str):
            set_error()
        if ret != tsigstr:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".tsig_keydata()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.tsig_keydata()
        if ret != None:
            set_error()
    except:
        set_error()
    tkdstr = "Twas brillig, and the slithy toves Did gyre and gimble in the wabe; All mimsy were the borogoves, And the mome raths outgrabe."
    resolver.set_tsig_keydata(tkdstr)
    try:
        ret = resolver.tsig_keydata()
        if not isinstance(ret, str):
            set_error()
        if ret != tkdstr:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".tsig_keyname()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    try:
        ret = resolver.tsig_keyname()
        if ret != None:
            set_error()
    except:
        set_error()
    tknstr = "key 2"
    resolver.set_tsig_keyname(tknstr)
    try:
        ret = resolver.tsig_keyname()
        if not isinstance(ret, str):
            set_error()
        if ret != tknstr:
            set_error()
    except:
        set_error()


#if not error_detected:
if True:
    method_name = class_name + ".usevc()"
    resolver = ldns.ldns_resolver.new_frm_file("/etc/resolv.conf")
    resolver.set_usevc(False)
    try:
        ret = resolver.usevc()
        if not isinstance(ret, bool):
            set_error()
        if ret != False:
            set_error()
    except:
        set_error()
    resolver.set_usevc(True)
    try:
        ret = resolver.usevc()
        if not isinstance(ret, bool):
            set_error()
        if ret != True:
            set_error()
    except:
        set_error()


if not error_detected:
    sys.stdout.write("%s: passed.\n" % (os.path.basename(__file__)))
else:
    sys.stdout.write("%s: errors detected.\n" % (os.path.basename(__file__)))
    sys.exit(1)



import ldns
import sys

if len(sys.argv) <= 1:
    print("Usage: %s zone_file" % sys.argv[0])
    sys.exit()

inp = open(sys.argv[1],"r");
# variables that preserve the parsers state
my_ttl = 3600;
my_origin = None
my_prev = None
# additional state variables
last_pos = 0
line_nr = 0

while True:
    ret = ldns.ldns_rr_new_frm_fp_l_(inp, my_ttl, my_origin, my_prev)
    s, rr, line_inc, new_ttl, new_origin, new_prev = ret  # unpack the result
    line_nr += line_inc # increase number of parsed lines
    my_prev = new_prev  # update ref to previous owner

    if s == ldns.LDNS_STATUS_SYNTAX_TTL:
        my_ttl = new_ttl  # update default TTL
        print("$TTL:", my_ttl)
    elif s == ldns.LDNS_STATUS_SYNTAX_ORIGIN:
        my_origin = new_origin  # update reference to origin
        print("$ORIGIN:", my_origin)
    elif s == ldns.LDNS_STATUS_SYNTAX_EMPTY:
        if last_pos == inp.tell():
            break  # no advance since last read - EOF
        last_pos = inp.tell()
    elif s != ldns.LDNS_STATUS_OK:
        print("! parse error in line", line_nr)
    else:
        # we are sure to have LDNS_STATUS_OK
        print(rr)

inp.close()
print("--------------------")
print("Read %d lines" % line_nr)
    

